@@ -2,7 +2,7 @@ dispBinTrend <- function(y, design=NULL, offset=NULL, df=5, span=0.3, min.n=400,
 #	Estimate common dispersion in bins based on AveLogCPM,
 #	then fit a curve through the dispersions
 #	Davis McCarthy, Gordon Smyth
-#	Created 10 Feb 2011.  Last modified 25 Nov 2013.
+#	Created 10 Feb 2011.  Last modified 1 Jun 2020.
 {
 #	Check y
 	y <- as.matrix(y)
@@ -30,7 +30,7 @@ dispBinTrend <- function(y, design=NULL, offset=NULL, df=5, span=0.3, min.n=400,
 	method.trend <- match.arg(method.trend, c("spline", "loess"))
 
 #	Check AveLogCPM
-	if(is.null(AveLogCPM)) AveLogCPM <- aveLogCPM(y,weights=weights)
+	if(is.null(AveLogCPM)) AveLogCPM <- aveLogCPM(y, offset=offset, weights=weights)
 
 #	Define bins of genes based on min.n in each bin
 #	All zero rows are marked as group==0
@@ -57,8 +57,9 @@ dispBinTrend <- function(y, design=NULL, offset=NULL, df=5, span=0.3, min.n=400,
 	bin.d <- bin.A <- rep(0,nbins)
 	for(i in 1:nbins) {
 		bin <- group==i
-		bin.d[i] <- estimateGLMCommonDisp(y[bin,], design, method=method.bin, offset[bin,], min.row.sum=0, weights=weights[bin,] ,...)
-		bin.A[i] <- mean(AveLogCPM[bin])
+		binAve <- AveLogCPM[bin]
+		bin.d[i] <- estimateGLMCommonDisp(y[bin,], design, method=method.bin, offset[bin,], min.row.sum=0, weights=weights[bin,], AveLogCPM=binAve, ...)
+		bin.A[i] <- mean(binAve)
 	}
 
 #	If just one bin, trended dispersion is constant
@@ -69,7 +70,7 @@ dispBinTrend <- function(y, design=NULL, offset=NULL, df=5, span=0.3, min.n=400,
 
 #	If few bins, use linear interpolation
 	if(nbins<7) {
-		f <- approxfun(bin.A,sqrt(bin.d),rule=2)
+		f <- approxfun(bin.A, sqrt(bin.d), rule=2, ties=mean)
 		dispersion <- f(AveLogCPM)^2
 		return(list(AveLogCPM=AveLogCPM, dispersion=dispersion, bin.AveLogCPM=bin.A, bin.dispersion=bin.d))
 	}
@@ -78,7 +79,7 @@ dispBinTrend <- function(y, design=NULL, offset=NULL, df=5, span=0.3, min.n=400,
 	if( method.trend=="spline" ) {
 		if(!requireNamespace("splines",quietly=TRUE)) stop("splines required but is not available")
 		p1 <- (1:(df-1))/df
-		knots1 <- quantile(bin.A,p=p1)
+		knots1 <- quantile(bin.A,probs=p1)
 		r <- range(bin.A)
 		knots2 <- r[1]+p1*(r[2]-r[1])
 		knots <- 0.3*knots1+0.7*knots2
@@ -91,7 +92,7 @@ dispBinTrend <- function(y, design=NULL, offset=NULL, df=5, span=0.3, min.n=400,
 #	Loess smoother though binned dispersions
 	if( method.trend=="loess" ) {
 		fit <- loessFit(sqrt(bin.d), bin.A, span=span, iterations=1)
-		f <- approxfun(bin.A, fit$fitted, rule=2)
+		f <- approxfun(bin.A, fit$fitted, rule=2, ties=mean)
 		dispersion <- f(AveLogCPM)^2
 	}
 
