@@ -3,14 +3,21 @@
 estimateDisp <- function(y, ...)
 UseMethod("estimateDisp")
 
-estimateDisp.DGEList <- function(y, design=NULL, prior.df=NULL, trend.method="locfit", mixed.df=FALSE, tagwise=TRUE, span=NULL, min.row.sum=5, grid.length=21, grid.range=c(-10,10), robust=FALSE, winsor.tail.p=c(0.05,0.1), tol=1e-06, ...)
-#  Yunshun Chen. Created 16 March 2016.
+estimateDisp.DGEList <- function(y, design=NULL, prior.df=NULL, trend.method="locfit", tagwise=TRUE, span=NULL, min.row.sum=5, grid.length=21, grid.range=c(-10,10), robust=FALSE, winsor.tail.p=c(0.05,0.1), tol=1e-06, ...)
+#  Yunshun Chen.
+#  Created 16 March 2016. Last modified 16 Oct 2019.
 {
 	y <- validDGEList(y)
 	group <- y$samples$group
 	lib.size <- y$samples$lib.size * y$samples$norm.factors
 
-	d <- estimateDisp(y=y$counts, design=design, group=group, lib.size=lib.size, offset=getOffset(y), prior.df=prior.df, trend.method=trend.method, mixed.df=mixed.df, tagwise=tagwise, span=span, min.row.sum=min.row.sum, grid.length=grid.length, grid.range=grid.range, robust=robust, winsor.tail.p=winsor.tail.p, tol=tol, weights=y$weights, ...)
+	if(is.null(design)) {
+		design <- y$design
+	} else {
+		y$design <- design
+	}
+
+	d <- estimateDisp(y=y$counts, design=design, group=group, lib.size=lib.size, offset=getOffset(y), prior.df=prior.df, trend.method=trend.method, tagwise=tagwise, span=span, min.row.sum=min.row.sum, grid.length=grid.length, grid.range=grid.range, robust=robust, winsor.tail.p=winsor.tail.p, tol=tol, weights=y$weights, ...)
 
 	y$common.dispersion <- d$common.dispersion
 	y$trended.dispersion <- d$trended.dispersion
@@ -23,19 +30,31 @@ estimateDisp.DGEList <- function(y, design=NULL, prior.df=NULL, trend.method="lo
 	y
 }
 
-estimateDisp.default <- function(y, design=NULL, group=NULL, lib.size=NULL, offset=NULL, prior.df=NULL, trend.method="locfit", mixed.df=FALSE, tagwise=TRUE, span=NULL, min.row.sum=5, grid.length=21, grid.range=c(-10,10), robust=FALSE, winsor.tail.p=c(0.05,0.1), tol=1e-06, weights=NULL, ...)
-#  Use GLM approach if a design matrix is given, and classic approach otherwise.
-#  It calculates a matrix of likelihoods for each gene at a set of dispersion grid points, and then calls WLEB() to do the shrinkage.
-#  Yunshun Chen, Gordon Smyth. Created July 2012. Last modified 18 March 2016.
+estimateDisp.SummarizedExperiment <- function(y, design=NULL, prior.df=NULL, trend.method="locfit", tagwise=TRUE, span=NULL, min.row.sum=5, grid.length=21, grid.range=c(-10,10), robust=FALSE, winsor.tail.p=c(0.05,0.1), tol=1e-06, ...)
+#  Yunshun Chen.
+#  Created 19 March 2020. Last modified 19 March 2020.
+{
+	y <- SE2DGEList(y)
+	y <- estimateDisp.DGEList(y, design=design, prior.df=prior.df, trend.method=trend.method, tagwise=tagwise, span=span, min.row.sum=min.row.sum, grid.length=grid.length, grid.range=grid.range, robust=robust, winsor.tail.p=winsor.tail.p, tol=tol, ...)
+	y
+}
+
+estimateDisp.default <- function(y, design=NULL, group=NULL, lib.size=NULL, offset=NULL, prior.df=NULL, trend.method="locfit", tagwise=TRUE, span=NULL, min.row.sum=5, grid.length=21, grid.range=c(-10,10), robust=FALSE, winsor.tail.p=c(0.05,0.1), tol=1e-06, weights=NULL, ...)
+#  Estimate common, trended and tagwise dispersions
+#  Use GLM approach if design matrix is given and classic approach otherwise.
+#  A matrix of likelihoods is computed for each gene at a set of dispersion grid points
+#  and WLEB() is called for weighted likelihood empirical Bayes.
+#  Yunshun Chen, Aaron Lun, Gordon Smyth.
+#  Created July 2012. Last modified 16 Oct 2019.
 {
 #	Check y
 	y <- as.matrix(y)
 	ntags <- nrow(y)
-	if(ntags==0) return(numeric(0))
+	if(ntags==0) return(list(span=span, prior.df=prior.df, prior.n=prior.n))
 	nlibs <- ncol(y)
 	
 #	Check trend.method
-	trend.method <- match.arg(trend.method, c("none", "loess", "locfit", "movingave"))
+	trend.method <- match.arg(trend.method, c("none", "loess", "locfit", "movingave", "locfit.mixed"))
 	
 #	Check group
 	if(is.null(group)) group <- rep(1, nlibs)
@@ -47,14 +66,19 @@ estimateDisp.default <- function(y, design=NULL, group=NULL, lib.size=NULL, offs
 	if(length(lib.size)!=nlibs) stop("Incorrect length of lib.size.")
 	
 #	Check offset
-	if(is.null(offset)) offset <- log(lib.size)
-	offset <- expandAsMatrix(offset, dim(y))
+	offset <- .compressOffsets(y, lib.size=lib.size, offset=offset)
+
+#	Check weights
+	weights <- .compressWeights(y, weights)
 
 #	Check for genes with small counts
 	sel <- rowSums(y) >= min.row.sum
+	sely <- .subsetMatrixWithoutCopying(y, i=sel)
+	seloffset <- .subsetMatrixWithoutCopying(offset, i=sel)
+	selweights <- .subsetMatrixWithoutCopying(weights, i=sel)
 	
 #	Spline points
-	spline.pts <- seq(from=grid.range[1], to=grid.range[2], length=grid.length)
+	spline.pts <- seq(from=grid.range[1], to=grid.range[2], length.out=grid.length)
 	spline.disp <- 0.1 * 2^spline.pts
 	grid.vals <- spline.disp/(1+spline.disp)
 	l0 <- matrix(0, sum(sel), grid.length)
@@ -69,7 +93,7 @@ estimateDisp.default <- function(y, design=NULL, group=NULL, lib.size=NULL, offs
 			design <- model.matrix(~group)
 		if( all(tabulate(group)<=1) ) {
 			warning("There is no replication, setting dispersion to NA.")
-			return(list(common.dispersion=NA, trended.dispersion=NA, tagwise.dispersion=NA))
+			return(list(common.dispersion=NA_real_, trended.dispersion=NA_real_, tagwise.dispersion=NA_real_))
 		}
 		
 		eq <- equalizeLibSizes(y, group=group, dispersion=0.01, lib.size=lib.size)
@@ -91,17 +115,19 @@ estimateDisp.default <- function(y, design=NULL, group=NULL, lib.size=NULL, offs
 		design <- as.matrix(design)
 		if(ncol(design) >= nlibs) {
 			warning("No residual df: setting dispersion to NA")
-			return(list(common.dispersion=NA, trended.dispersion=NA, tagwise.dispersion=NA))
+			return(list(common.dispersion=NA_real_, trended.dispersion=NA_real_, tagwise.dispersion=NA_real_))
 		}
 
-		# Protect against zeros.
-		glmfit <- glmFit(y[sel,], design, offset=offset[sel,], dispersion=0.05, prior.count=0)
-		zerofit <- (glmfit$fitted.values < 1e-4) & (glmfit$counts < 1e-4)
+		# Identify which observations have means of zero (weights aren't needed here).
+		glmfit <- glmFit(sely, design, offset=seloffset, dispersion=0.05, prior.count=0)
+		zerofit <- (glmfit$counts < 1e-4 & glmfit$fitted.values < 1e-4)
 		by.group <- .comboGroups(zerofit)
 
 		for (subg in by.group) { 
 			cur.nzero <- !zerofit[subg[1],]
 			if (!any(cur.nzero)) { next } 
+
+			# Removing samples with zero means from design, to avoid attempts to converge to -Inf.
 			if (all(cur.nzero)) { 
 				redesign <- design
 			} else {
@@ -111,38 +137,54 @@ estimateDisp.default <- function(y, design=NULL, group=NULL, lib.size=NULL, offs
 				if (nrow(redesign) == ncol(redesign)) { next }
 			}
 
+			cury <- .subsetMatrixWithoutCopying(sely, i=subg, j=cur.nzero)
+			curo <- .subsetMatrixWithoutCopying(seloffset, i=subg, j=cur.nzero)
+			curw <- .subsetMatrixWithoutCopying(selweights, i=subg, j=cur.nzero)
+
 			# Using the last fit to hot-start the next fit
 			last.beta <- NULL
-			for(i in 1:grid.length) {
-				out <- adjustedProfileLik(spline.disp[i], y=y[sel, ][subg,cur.nzero,drop=FALSE], design=redesign, 
-					offset=offset[sel,][subg,cur.nzero,drop=FALSE], start=last.beta, get.coef=TRUE)
+			for(i in seq_len(grid.length)) {
+				out <- adjustedProfileLik(spline.disp[i], y=cury, design=redesign, 
+					offset=curo, weights=curw, start=last.beta, get.coef=TRUE)
 				l0[subg,i] <- out$apl
 				last.beta <- out$beta
 			}
 		}
 	}
 
+	# Calculate common dispersion
 	overall <- maximizeInterpolant(spline.pts, matrix(colSums(l0), nrow=1))
 	common.dispersion <- 0.1 * 2^overall
-	AveLogCPM <- aveLogCPM(y, lib.size=lib.size, dispersion=common.dispersion, weights=weights)
-
-	out.1 <- WLEB(theta=spline.pts, loglik=l0, covariate=AveLogCPM[sel], trend.method=trend.method, 
-		mixed.df=mixed.df, span=span, overall=FALSE, individual=FALSE, m0.out=TRUE)
-	disp.trend <- 0.1 * 2^out.1$trend
-	trended.dispersion <- rep( disp.trend[which.min(AveLogCPM[sel])], ntags )
-	trended.dispersion[sel] <- disp.trend
 
+	# Allow dispersion trend?
+	if(trend.method!="none"){
+		AveLogCPM <- aveLogCPM(y, lib.size=lib.size, dispersion=common.dispersion, weights=weights)
+		out.1 <- WLEB(theta=spline.pts, loglik=l0, covariate=AveLogCPM[sel], trend.method=trend.method, 
+			span=span, overall=FALSE, individual=FALSE, m0.out=TRUE)
+		span <- out.1$span
+		m0 <- out.1$shared.loglik
+		disp.trend <- 0.1 * 2^out.1$trend
+		trended.dispersion <- rep( disp.trend[which.min(AveLogCPM[sel])], ntags )
+		trended.dispersion[sel] <- disp.trend
+	} else {
+		AveLogCPM <- NULL
+		m0 <- matrix(colMeans(l0), ntags, length(spline.pts), byrow=TRUE)
+		disp.trend <- common.dispersion
+		trended.dispersion <- NULL
+	}
+	
+	# Are tagwise dispersions required?
 	if(!tagwise) return(list(common.dispersion=common.dispersion, trended.dispersion=trended.dispersion))
 
 	# Calculate prior.df
 	if(is.null(prior.df)){
-		glmfit <- glmFit(y[sel,], design, offset=offset[sel,], dispersion=disp.trend, prior.count=0)
+		glmfit <- glmFit(sely, offset=seloffset, weights=selweights, design=design, dispersion=disp.trend, prior.count=0)
 
 		# Residual deviances
 		df.residual <- glmfit$df.residual
 
 		# Adjust df.residual for fitted values at zero
-		zerofit <- (glmfit$fitted.values < 1e-4) & (glmfit$counts < 1e-4)
+		zerofit <- (glmfit$counts < 1e-4 & glmfit$fitted.values < 1e-4)
 		df.residual <- .residDF(zerofit, design)
 
 		# Empirical Bayes squeezing of the quasi-likelihood variance factors
@@ -155,11 +197,11 @@ estimateDisp.default <- function(y, design=NULL, group=NULL, lib.size=NULL, offs
 	ncoefs <- ncol(design)
 	prior.n <- prior.df/(nlibs-ncoefs)
 
-	if (trend.method!='none') { 
+	# Initiate tagwise dispersions
+	if(trend.method!="none")
 		tagwise.dispersion <- trended.dispersion
-	} else {
+	else
 		tagwise.dispersion <- rep(common.dispersion, ntags)
-	}
 
 	# Checking if the shrinkage is near-infinite.
 	too.large <- prior.n > 1e6
@@ -169,9 +211,9 @@ estimateDisp.default <- function(y, design=NULL, group=NULL, lib.size=NULL, offs
 			temp.n[too.large] <- 1e6 
 		}
 
-		# Estimating tagwise dispersions	
+		# Estimating tagwise dispersions
 		out.2 <- WLEB(theta=spline.pts, loglik=l0, prior.n=temp.n, covariate=AveLogCPM[sel], 
-			trend.method=trend.method, mixed.df=mixed.df, span=span, overall=FALSE, trend=FALSE, m0=out.1$shared.loglik)
+			trend.method=trend.method, span=span, overall=FALSE, trend=FALSE, m0=m0)
 
 		if (!robust) { 
 			tagwise.dispersion[sel] <- 0.1 * 2^out.2$individual
@@ -188,16 +230,16 @@ estimateDisp.default <- function(y, design=NULL, group=NULL, lib.size=NULL, offs
 		prior.n[sel] <- temp.n
 	}
 
-	list(common.dispersion=common.dispersion, trended.dispersion=trended.dispersion, tagwise.dispersion=tagwise.dispersion, span=out.1$span, prior.df=prior.df, prior.n=prior.n)
+	list(common.dispersion=common.dispersion, trended.dispersion=trended.dispersion, tagwise.dispersion=tagwise.dispersion, span=span, prior.df=prior.df, prior.n=prior.n)
 }
 
 
-WLEB <- function(theta, loglik, prior.n=5, covariate=NULL, trend.method="locfit", mixed.df=FALSE, span=NULL, 
+WLEB <- function(theta, loglik, prior.n=5, covariate=NULL, trend.method="locfit", span=NULL, 
 	overall=TRUE, trend=TRUE, individual=TRUE, m0=NULL, m0.out=FALSE)
 #  Weighted likelihood empirical Bayes for estimating a parameter vector theta
 #  given log-likelihood values on a grid of theta values
 #  Yunshun Chen, Gordon Smyth
-#	Created July 2012. Last modified 24 October 2012.
+#	Created July 2012. Last modified 16 Oct 2019.
 {
 #	Check loglik
 	loglik <- as.matrix(loglik)
@@ -208,9 +250,7 @@ WLEB <- function(theta, loglik, prior.n=5, covariate=NULL, trend.method="locfit"
 	if(is.null(covariate))
 		trend.method <- "none"
 	else
-		trend.method <- match.arg(trend.method, c("none", "loess", "locfit", "movingave"))
-
-	if(trend.method=="locfit" & mixed.df) trend.method <- "locfit_mix"
+		trend.method <- match.arg(trend.method, c("none", "loess", "locfit", "movingave", "locfit.mixed"))
 
 #	Set span
 	if(is.null(span)) if(ntags<=50) span <- 1 else span <- 0.25+0.75*(50/ntags)^0.5
@@ -233,7 +273,7 @@ WLEB <- function(theta, loglik, prior.n=5, covariate=NULL, trend.method="locfit"
 		},
 		"loess" = loessByCol(loglik, covariate, span=span)$fitted.values,
 		"locfit" = locfitByCol(loglik, covariate, span=span, degree=0),
-		"locfit_mix" = {
+		"locfit.mixed" = {
 			deg0 <- locfitByCol(loglik, covariate, span=span, degree=0)
 			deg1 <- locfitByCol(loglik, covariate, span=span, degree=1)
 			r <- range(covariate)
@@ -244,7 +284,7 @@ WLEB <- function(theta, loglik, prior.n=5, covariate=NULL, trend.method="locfit"
 	)
 
 #	make sure each row of m0 is unimodal
-	if(trend.method=="locfit_mix"){
+	if(trend.method=="locfit.mixed"){
 		for(i in ncol(m0):3){
 			diff1_m0 <- m0[,i] - m0[,i-1]
 			diff2_m0 <- m0[,i-1] - m0[,i-2]
@@ -267,3 +307,39 @@ WLEB <- function(theta, loglik, prior.n=5, covariate=NULL, trend.method="locfit"
 
 	out
 }
+
+.subsetMatrixWithoutCopying <- function(x, i, j) 
+# This will attempt to subset the matrix without any copying if
+# it detects that 'i' and 'j' don't modify the ordering of the matrix.
+# This reduces the memory footprint for large matrices.
+#
+# written by Aaron Lun
+# created 29 September 2016
+# last modified 16 December 2018
+{
+	isokay <- TRUE
+	if (!missing(i)) {
+		# Most flexible way of handling different types of subset vectors;
+		# try it out and see if it gives the same results.
+		example <- cbind(seq_len(nrow(x)))
+		rownames(example) <- rownames(x)
+		if (!identical(example, example[i,,drop=FALSE])) isokay <- FALSE
+	}
+	if (!missing(j)) {
+		example <- rbind(seq_len(ncol(x)))
+		colnames(example) <- colnames(x)
+		if (!identical(example, example[,j,drop=FALSE])) isokay <- FALSE
+	}
+
+	if (isokay) {
+		# Avoids copying if no modification incurred.
+		return(x)
+	} else if (!missing(i) && !missing(j)) {
+		return(x[i,j,drop=FALSE])
+	} else if (!missing(i)) {
+		return(x[i,,drop=FALSE])
+	} else {
+		return(x[,j,drop=FALSE])
+	}
+}
+
