@@ -4,14 +4,24 @@ glmFit <- function(y, ...)
 UseMethod("glmFit")
 
 glmFit.DGEList <- function(y, design=NULL, dispersion=NULL, prior.count=0.125, start=NULL, ...)
-#	Created 11 May 2011.  Last modified 11 March 2013.
+#	Created 11 May 2011.  Last modified 21 Nov 2018.
 {
+#	The design matrix defaults to the oneway layout defined by y$samples$group.
+#	If there is only one group, then the design matrix is left NULL so that a
+#	matrix with a single intercept column will be set later by glmFit.default.
+	if(is.null(design)) {
+		design <- y$design
+		if(is.null(design)) {
+			group <- droplevels(as.factor(y$samples$group))
+			if(nlevels(group) > 1L) design <- model.matrix(~y$samples$group)
+		}
+	}
 	if(is.null(dispersion)) dispersion <- getDispersion(y)
 	if(is.null(dispersion)) stop("No dispersion values found in DGEList object.")
-
+	offset <- getOffset(y)
 	if(is.null(y$AveLogCPM)) y$AveLogCPM <- aveLogCPM(y)
 
-	fit <- glmFit(y=y$counts,design=design,dispersion=dispersion,offset=getOffset(y),lib.size=NULL,weights=y$weights,prior.count=prior.count,start=start,...)
+	fit <- glmFit(y=y$counts,design=design,dispersion=dispersion,offset=offset,lib.size=NULL,weights=y$weights,prior.count=prior.count,start=start,...)
 	fit$samples <- y$samples
 	fit$genes <- y$genes
 	fit$prior.df <- y$prior.df
@@ -19,40 +29,66 @@ glmFit.DGEList <- function(y, design=NULL, dispersion=NULL, prior.count=0.125, s
 	new("DGEGLM",fit)
 }
 
+glmFit.SummarizedExperiment <- function(y, design=NULL, dispersion=NULL, prior.count=0.125, start=NULL, ...)
+#	Created 19 March 2020.  Last modified 19 March 2020.
+{
+	y <- SE2DGEList(y)
+	glmFit.DGEList(y, design=design, dispersion=dispersion, prior.count=prior.count, start=start, ...)
+}
+
 glmFit.default <- function(y, design=NULL, dispersion=NULL, offset=NULL, lib.size=NULL, weights=NULL, prior.count=0.125, start=NULL, ...)
 #	Fit negative binomial generalized linear model for each transcript
 #	to a series of digital expression libraries
-#	Davis McCarthy and Gordon Smyth
-#	Created 17 August 2010. Last modified 18 May 2015.
+#	Davis McCarthy, Gordon Smyth, Yunshun Chen, Aaron Lun
+#	Created 17 August 2010. Last modified 7 Aug 2019.
 {
 #	Check y
 	y <- as.matrix(y)
+	if(mode(y) != "numeric") stop("y is not a numeric matrix")
 	ntag <- nrow(y)
 	nlib <- ncol(y)
 
 #	Check design
 	if(is.null(design)) {
-		design <- matrix(1,ncol(y),1)
+		design <- matrix(1,nlib,1)
 		rownames(design) <- colnames(y)
 		colnames(design) <- "Intercept"
 	} else {
 		design <- as.matrix(design)
+		if(nrow(design) != nlib) stop("nrow(design) disagrees with ncol(y)")
 		ne <- nonEstimable(design)
 		if(!is.null(ne)) stop(paste("Design matrix not of full rank.  The following coefficients not estimable:\n", paste(ne, collapse = " ")))
 	}
 
 #	Check dispersion
 	if(is.null(dispersion)) stop("No dispersion values provided.")
-	dispersion.mat <- expandAsMatrix(dispersion, dim(y), byrow=FALSE)
-
-#	Check offset and lib.size
-	if(is.null(offset)) {
-		if(is.null(lib.size)) lib.size <- colSums(y)
-		if(any(lib.size==0L)) stop("Zero library size detected. Remove the empty libraries before proceeding to the next step.")
-		offset <- log(lib.size)
+	if(anyNA(dispersion)) stop("NA dispersions not allowed")
+	if(!is.numeric(dispersion)) stop("dispersion must be numeric")
+	if(is.null(dim(dispersion))) {
+		if( !any(length(dispersion)==c(1L,ntag)) ) stop("dispersion has wrong length. As a vector, it should agree with nrow(y)")
+	} else {
+		if( !all(dim(dispersion)==dim(y)) ) stop("Dimensions of dispersion don't agree with dimensions of y")
 	}
-	if(any(is.infinite(offset))) stop("Infinite offset value detected. Check the input offset.")
-	offset <- expandAsMatrix(offset,dim(y))
+	dispersion.mat <- .compressDispersions(y, dispersion)
+
+#	Check offset
+	if(!is.null(offset)) {
+		if(!is.numeric(offset)) stop("offset must be numeric")
+		if(is.null(dim(offset))) {
+			if( !any(length(offset)==c(1L,nlib)) ) stop("offset has wrong length. As a vector, it should agree with ncol(y)")
+		} else {
+			if( !all(dim(offset)==dim(y)) ) stop("Dimensions of offset don't agree with dimensions of y")
+		}
+	}
+
+#	Check lib.size
+	if(!is.null(lib.size)) {
+		if(!is.numeric(lib.size)) stop("lib.size must be numeric")
+		if( !any(length(lib.size)==c(1L,nlib)) ) stop("lib.size has wrong length, should agree with ncol(y)")
+	}
+
+#	Comsolidate lib.size and offset into a compressed matrix
+	offset <- .compressOffsets(y=y, lib.size=lib.size, offset=offset)
 
 #	weights are checked in lower-level functions
 
@@ -60,7 +96,7 @@ glmFit.default <- function(y, design=NULL, dispersion=NULL, offset=NULL, lib.siz
 #	If the design is equivalent to a oneway layout, use a shortcut algorithm
 	group <- designAsFactor(design)
 	if(nlevels(group)==ncol(design)) {
-		fit <- mglmOneWay(y,design=design,dispersion=dispersion.mat,offset=offset,weights=weights,coef.start=start)
+		fit <- mglmOneWay(y,design=design,group=group,dispersion=dispersion.mat,offset=offset,weights=weights,coef.start=start)
 		fit$deviance <- nbinomDeviance(y=y,mean=fit$fitted.values,dispersion=dispersion.mat,weights=weights)
 		fit$method <- "oneway"
 	} else {
@@ -93,7 +129,7 @@ glmFit.default <- function(y, design=NULL, dispersion=NULL, offset=NULL, lib.siz
 glmLRT <- function(glmfit,coef=ncol(glmfit$design),contrast=NULL)
 #	Tagwise likelihood ratio tests for DGEGLM
 #	Gordon Smyth, Davis McCarthy and Yunshun Chen.
-#	Created 1 July 2010.  Last modified 11 June 2015.
+#	Created 1 July 2010.  Last modified 31 Oct 2017.
 {
 #	Check glmfit
 	if(!is(glmfit,"DGEGLM")) {
@@ -127,14 +163,14 @@ glmLRT <- function(glmfit,coef=ncol(glmfit$design),contrast=NULL)
 		logFC <- glmfit$coefficients[,coef,drop=FALSE]/log(2)
 	} else {
 		contrast <- as.matrix(contrast)
+		if(nrow(contrast) != ncol(glmfit$coefficients)) stop("contrast vector of wrong length, should be equal to number of coefficients in the linear model.")
 		qrc <- qr(contrast)
 		ncontrasts <- qrc$rank
 		if(ncontrasts==0) stop("contrasts are all zero")
 		coef <- 1:ncontrasts
-		if(ncontrasts < ncol(contrast)) contrast <- contrast[,qrc$pivot[coef]]
-		logFC <- drop((glmfit$coefficients %*% contrast)/log(2))
+		logFC <- (glmfit$coefficients %*% contrast)/log(2)
 		if(ncontrasts>1) {
-			coef.name <- paste("LR test of",ncontrasts,"contrasts")
+			coef.name <- paste("LR test on",ncontrasts,"degrees of freedom")
 		} else {
 			contrast <- drop(contrast)
 			i <- contrast!=0
@@ -145,7 +181,7 @@ glmLRT <- function(glmfit,coef=ncol(glmfit$design),contrast=NULL)
 		Q <- qr.Q(qrc,complete=TRUE,Dvec=Dvec)
 		design <- design %*% Q
 	}
-	if(length(coef)==1) logFC <- as.vector(logFC)
+	if(length(coef)==1) logFC <- drop(logFC)
 
 #	Null design matrix
 	design0 <- design[,-coef,drop=FALSE]
